@@ -10,6 +10,7 @@ use Mojo::Base 'Mojolicious::Controller';
 use Mojo::JSON 'j';
 use Convos::Core::Commands;
 use constant PING_INTERVAL => $ENV{CONVOS_PING_INTERVAL} || 30;
+use constant DEFAULT_RESPONSE => "Hey, I don't know how to respond to that. Try /help to see what I can so far.";
 
 =head1 METHODS
 
@@ -26,9 +27,9 @@ sub socket {
   my($sub, $tid);
 
   Mojo::IOLoop->stream($self->tx->connection)->timeout(PING_INTERVAL * 2);
+  Scalar::Util::weaken($self);
 
   # send ping frames
-  Scalar::Util::weaken($self);
   $tid = Mojo::IOLoop->recurring(PING_INTERVAL, sub {
            $self->send('<div class="ping"/>');
          });
@@ -38,22 +39,17 @@ sub socket {
     message => sub {
       my ($self, $octets) = @_;
       my $dom = Mojo::DOM->new($octets)->at('div');
-      return if $dom && $dom->attr('class') eq 'pong';
       $self->logf(debug => '[ws] < %s', $octets);
 
-      if($dom and $dom->{'id'} and $dom->{'data-server'}) {
-        @$dom{qw/ server target uuid /} = map { delete $dom->{$_} || '' } qw/ data-server data-target id /;
+      if($dom and $dom->attr('class') eq 'pong') {
+        return;
+      }
+      elsif($dom and $dom->{'id'} and $dom->{'data-server'}) {
+        @$dom{qw( server target uuid )} = map { delete $dom->{$_} || '' } qw( data-server data-target id );
         $self->_handle_socket_data($dom);
       }
       else{
-        $self->send_partial('event/server_message',
-          status => 400,
-          server => $dom->{'data-server'} || 'any',
-          message => "Invalid message ($octets)",
-          status => 400,
-          timestamp => time,
-          uuid => '',
-        )->finish;
+        $self->_send_400($dom, "Invalid message ($octets)")->finish;
       }
     }
   );
@@ -61,7 +57,7 @@ sub socket {
     finish => sub {
       my $self = shift or return;
       Mojo::IOLoop->remove($tid);
-      delete $self->stash->{$_} for qw/ sub redis /;
+      delete $self->stash->{$_} for qw( sub redis );
     }
   );
 
@@ -90,27 +86,52 @@ sub socket {
   );
 }
 
+sub _convos_message {
+  my($self, $args, $input, $response) = @_;
+  my $login = $self->session('login');
+
+  $self->send_partial(
+    'event/message',
+    highlight => 0,
+    message => $input,
+    nick =>  $login,
+    server => $args->{server},
+    status => 200,
+    target => '',
+    timestamp => time,
+    uuid => $args->{uuid}.'_',
+  );
+  $self->send_partial(
+    'event/message',
+    highlight => 0,
+    message => $response,
+    nick => $args->{server},
+    server => $args->{server},
+    status => 200,
+    target => '',
+    timestamp => time,
+    uuid => $args->{uuid},
+  );
+}
+
 sub _handle_socket_data {
   my ($self, $dom) = @_;
   my $cmd = Mojo::Util::html_unescape($dom->text(0));
   my $login = $self->session('login');
 
-  if ($cmd =~ s!^/(\w+)\s*(.*)!!) {
+  if($cmd =~ s!^/(\w+)\s*(.*)!!) {
     my($action, $arg) = ($1, $2);
     $arg =~ s/\s+$//;
     if (my $code = Convos::Core::Commands->can($action)) {
       $cmd = $self->$code($arg, $dom);
     }
     else {
-      return $self->send_partial('event/server_message',
-        server => $dom->{server},
-        message => 'Unknown command. Type /help to see available commands.',
-        status => 400,
-        timestamp => time,
-        uuid => '',
-      );
+      return $self->_send_400($dom, 'Unknown command. Type /help to see available commands.');
     }
   }
+  elsif($dom->{server} eq 'convos') {
+    return $self->_convos_message($dom, $cmd, DEFAULT_RESPONSE);
+  }
   elsif($dom->{target}) {
     $cmd = "PRIVMSG $dom->{target} :$cmd";
   }
@@ -130,6 +151,19 @@ sub _handle_socket_data {
   }
 }
 
+sub _send_400 {
+  my($self, $args, $message) = @_;
+
+  $self->send_partial(
+    'event/server_message',
+    status => 400,
+    timestamp => time,
+    uuid => '',
+    server => $args->{'data-server'} || 'any',
+    message => $message,
+  );
+}
+
 =head1 COPYRIGHT
 
 See L<Convos>.
